#
# $Id: rc.tcl,v 1.13 2000/03/08 14:45:35 elias Exp $
# Copyright 1999 Security Dynamics
#
# Add and remove startup rc-script.
# Variant:
#

namespace eval ::CatRC { ;#

catch {namespace import ::CatDbg::*}

variable appdir "<<INSTALL:APPDIR>>"
variable vardir "<<INSTALL:VARDIR>>"



variable inittab /etc/inittab
variable rcdir /etc
variable level 2

proc Add_rc {script startname} {
   
     


    

    
    
    
    
    
    

    variable inittab
    variable rcdir
    variable level

    catch { Log "updating /etc/inittab to run $rcdir/rc.$startname"}

    if [file writable $rcdir] {
	if ![addentry $rcdir/rc.$startname $script] {
	    return 0
	}
	addline $inittab "agents:$level:wait:$rcdir/rc.$startname"
    } else {
	catch {Log "Don't have permission to write $rcdir"}
    }
    

    return 1
}

proc Remove_rc {startname} {
    
     
    

     


    

    variable inittab
    variable rcdir
    variable level

    catch { Log "updating /etc/inittab and/or $rcdir/rc.$startname" }

    if [file writable $rcdir] {
	if ![delentry $rcdir/rc.$startname] {
	    return 0
	}
	if ![file exists $rcdir/rc.$startname] {
	    delline $inittab "agents:$level:wait:$rcdir/rc.$startname"
	}
    } else {
	catch {Log "Don't have permission to write $rcdir"}
    }
    

    return 1
}


# Initialize or add entry to startup script
#
proc addentry {filename script} {
    variable appdir

    # Create a new file
    #
    if ![file exists $filename] {
	if [catch {file copy -- $script $filename} m] {
	    catch {Log "failed to copy rc-script; $m"}
	    return 0
	}
	catch {file attributes $filename -permissions 0744}
    }

    # Add an new entry to existing file
    #
    set startprg "$appdir/bin/start"
    set tmpfile "$filename.[pid]"
    if [catch {file rename -force $filename $tmpfile} error] {
	Log {Log "Cannot move $filename: $error"}
	return 0
    }
    if [catch {open $tmpfile {RDONLY}} fileId] {
	catch {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }
    set acc ""
    set done 0
    set inside 0
    while {[gets $fileId line] >= 0} {
	if !$done {
	    if $inside {
		if {[string compare $line "\""] == 0} {
		    set line "$startprg\n$line"
		    set done 1
		} else {
		    if {[string compare $line $startprg] == 0} {
			catch {Log "Current entry already present."}
			set done 1
		    }
		}
	    } else {
		if [regexp {^START=\"} $line] {
		    set inside 1
		}
	    }
	}
	append acc "$line\n"
    }
    close $fileId

    if [catch {open $tmpfile {WRONLY TRUNC}} fileId] {
	Log {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }

    puts -nonewline $fileId $acc
    close $fileId
    if [catch {file rename -force $tmpfile $filename} error] {
	Log {Log "Cannot move $tmpfile; $error"}
	catch {file delete -force -- $tmpfile}
	return 0
    }
    catch {Log "Adding starup entry"}
    catch {file attributes $filename -permissions 0744}

    return 1
}

# Remove entry from startup script, delete script when all entries are
# gone.
#
proc delentry {filename} {

    variable appdir

    if ![file exists $filename] {
	catch {Log "rc file $filename does not exist"}
	return 0
    }
    if ![file writable $filename] {
	catch {Log "rc file $filename is not writable"}
	return 0
    }

    set tmpfile $filename.[pid]
    set startprg "$appdir/bin/start"

    if [catch [file copy $filename $tmpfile] error] {
	catch {Log "Cannot copy $filename; $error"}
	return 0
    }

    if [catch {open $tmpfile {RDONLY}} fileId] {
	catch {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }

    set acc ""
    set done 0
    set inside 0
    set nentries 0
    while {[gets $fileId line] >= 0} {
	if !$done {
	    if $inside {
		if {[string compare $line $startprg] == 0} {
		    set line ""
		} elseif {[string compare $line "\""] == 0} {
		    set done 1
		} else {
		    incr nentries
		}
	    } else {
		if [regexp {^START=\"} $line] {
		    set inside 1
		}
	    }
	}
	if {[string compare $line ""] != 0} {
	    append acc "$line\n"
	}
    }

    # When all entries are deleted, remove file
    #
    if {$nentries == 0} {
	close $fileId
	catch {file delete -force -- $tmpfile}
	catch {file delete -force -- $filename}
    } else {
	close $fileId
	if [catch {open $tmpfile {WRONLY TRUNC}} fileId] {
	    Log {Log "Cannot open $tmpfile"}
	    catch {file delete -force -- $tmpfile}
	    
	    return 0
	}
	puts -nonewline $fileId $acc
	close $fileId
	if [catch {file rename -force $tmpfile $filename} error] {
	    Log {Log "Cannot move $tmpfile; $error"}
	    catch {file delete -force -- $tmpfile}

	    return 0
	}
    }
    return 1
}

proc findline {filename line} {
    if [catch {set f [open $filename]} msg] {
	catch {Log "Cannot read $filename"}
	return 0
    }
    set found 0
    foreach l [split [read $f] "\n"] {
	if {[string compare $line $l] == 0} {
	    incr found
	    break
	}
    }
    close $f

    return $found
}

proc addline {filename addline} {
    if [findline $filename $addline] {
	return 1
    }

    set tmpfile $filename.[pid]
    if [catch {file copy $filename $tmpfile} msg] {
	catch {Log "unable to copy $filename to $tmpfile: $msg"}
	return 0
    }
    if [catch {set f [open $tmpfile a]} msg] {
	catch {Log "unable to write $tmpfile: $msg"}
	catch {file delete -force $tmpfile}
	return 0
    }
    puts $f $addline
    close $f

    if [catch {file rename -force $tmpfile $filename} msg] {
	catch {Log "unable to write $filename: $msg"}
	catch {file delete -force $tmpfile}
	return 0
    }
    catch {file attributes $filename -permissions 0644}

    return 1
}

proc delline {filename delline} {
    if ![findline $filename $delline] {
	return 1
    }

    set tmpfile $filename.[pid]
    if [catch {set f [open $filename]} msg] {
	catch {Log "Cannot read $filename"}
	return 0
    }
    set lines [split [read $f] "\n"]
    close $f

    if [catch {set f [open $tmpfile w]} msg] {
	catch {Log "unable to write $tmpfile: $msg"}
	return 0
    }

    set found 0
    foreach line $lines {
	if {[string compare $line $delline] == 0} {
	    incr found
	    continue
	}
	if {$found && [string length $line] == 0} continue
	puts $f $line
    }
    close $f

    if [catch {file rename -force $tmpfile $filename} msg] {
	catch {Log "unable to write $filename: $msg"}
	catch {file delete -force $tmpfile}
	return 0
    }
    catch {file attributes $filename -permissions 0644}

    return 1
}


}; # end namespace CatRC
