#
# $Id: instutils.tcl,v 1.33 2000/01/16 19:43:59 mikko Exp $
# Copyright 1999 Security Dynamics
#
# Q&D installation utils
#
# Installation sources description files inside the ::CatInst namespace.
# Ideally, it should be done by a safe interpreter with a very limited
# number of procs (a subset of the ones in CatInst), and about half a
# dozen variables.
#
# Install of each module records file lists and module dependencies
# in the $VARDIR/install directory.
#
# Uninstallation reads file in said directory and deletes everything
# listed.  But first it checks dependencies...
#
# XXX: Uninstall removes entire directories.
#
# XXX: There is no way to have shared files or directories. These have
#      to be provided by some other module, with dependencies.
#
# Before anything else will work, you must do:
#
#  InstDir SRCDIR <path>		./module
#  InstDir APPDIR <absolute-path>	/opt/Product
#  InstDir ETCDIR <absolute-path>	/etc/opt/Product
#  InstDir VARDIR <absolute-path>	/var/opt/Product
#

rename exit original_exit

proc exit {{code 0}} {
    catch {CatInst::killer execute}
    original_exit $code
}

namespace eval ::CatInst { ;#

namespace export InstDir Install Uninstall InstallInfo IsInstalled
namespace export ListInstallables ListInstalled CleanupInstall
namespace export dsort lrev

catch {namespace import ::CatDbg::*}

variable APPDIR	  ;# target application (bin, lib) dir
variable VARDIR   ;# target var dir
variable ETCDIR   ;# target config dir
variable SRCDIR   ;# source dir (or dir above source dir)
variable DATADIR  ;# actual source dir
variable TARGET   ;# current fileset target dir
variable BRAND    ;# Brand name
variable SHORTNAME;# Short Brand name
variable instfile ;# installation record file for module
variable setname  ;# current module (file set) name
variable reqlist  ;# list of required modules
variable status   ;# status "cleanup" / anything else (see CleanupInstall)
variable MOD	  ;# Synonym for setname, for use in files
variable MODULENAME;# Description from fileset
variable isagent  ;# True if this is an agent
variable rmpaths  ;# Files in need of removal
variable unrmpaths;# Ditto that could not be removed, renamed or anything
variable EXPORT   ;# Export version 0/1

#----------------------------------------------------------------------
# Private functions

# Does not belong here...
#
proc get_file {name} {
    set f [open $name]
    set s [read $f]
    close $f
    return $s
}

# Reverse a list (does not belong here either)
#
proc lrev {list} {
    set result {}
    foreach e $list {
	set result [linsert $result 0 $e]
    }
    return $result
}

# dsort deplist --
#
#  Each element in deplist consists of <element> <list-of-dependencies>
#  Returns a list in dependency order (use lrev to get a list suitable
#  for "make"-type operations
#
proc dsort {lists {errvar {}}} {

    foreach {elem deps} $lists {
	lappend needs($elem) $deps
	set tmp($elem) 1
	foreach elem $deps {
	    set tmp($elem) 1
	}
    }
    set elems [array names tmp]

    set order {}
    while {[llength $elems] > 0} {
	catch { unset needed }
	foreach elem $elems {
	    if [info exists needs($elem)] {
		foreach n $needs($elem) {
		    foreach n $n {
			set needed($n) 1
		    }
		}
	    }
	}
	set next {}
	foreach elem $elems {
	    if ![info exists needed($elem)] {
		lappend order $elem
	    } else {
		lappend next $elem
	    }
	}
	if {[string compare $elems $next] == 0} {
	    if [string length $errvar] {
		uplevel $errvar err
		set err $elems
	    }
	    error "dsort loop in: $elems"
	}
	set elems $next
    }
    return $order
}

# Delete file or directory. If it does not want to, rename it
# to something "harmless".  If that cannot be done, somebody
# will have to come back with bigger guns...
#
# All names awaiting execution are appended to "rmpaths".
#
proc rm {path} {
    variable rmpaths

    D 3 "rm $path"
    if ![file exists $path] {
	return 1
    }
    if ![catch {file delete -force -- $path} msg] {
	Log "rm $path"
	return 1
    }
    Log "rm $path: $msg"
    lappend rmpaths $path

    return 1
}

proc killer {{mode execute}} {
    variable rmpaths
    variable APPDIR
    variable rm_id
    variable killer_tmp

    
    
    global rmpaths

    if {[string compare $mode execute] == 0} {
	if {[llength $rmpaths] == 0} {
		return 1
	}
	set name "/tmp/catkiller.[pid]"
	lappend rmpaths $name
	set script "# Connect agent cleanup script.\n"
	append script "exec > /tmp/killer.log 2>&1"
	append script "PATHS='\n"
	foreach p $rmpaths {
	    append script " $p\n"
	}
	append script "'\n"
	append script {
	    times=0
	    more=1
	    while [ $more -gt 0 -a $times -lt 10 ]; do
	    more=0
	    for p in $PATHS; do
	    rm -f "$p"
	    test -f "$p" -o -d "$p" && more=1
	    done
	    times=`expr $times + 1`
	    sleep $times
	    done
	}
	set f [open $name w]
	puts $f $script
	close $f
	exec sh $f >/dev/null 2>/dev/null &
    }
    
    
    return 1
}



# Add ourselves to some external database of installed stuff
#
proc register_software {install errorvar} {
    upvar $errorvar err
    variable MODULENAME
    variable MOD
    variable SHORTNAME
    variable APPDIR

    

    return 1
}

# Put together an error message as a list on the form
#
#  CODE message [extra-data]
#
proc err {varname code string {extra {}}} {
    upvar $varname err

    set err [list $code $string $extra]
    Log "Error: $err"
}

# Check is a string looks like a proper error list
#
proc proper_error {string} {
    if [catch {set code [lindex $string 0]}] {
	return 0
    }
    if {[llength $string] < 2} {
	return 0
    }
    if ![regexp {^[A-Z_0-9]+$} $code] {
	return 0
    }
    return 1
}

proc target_expand {pathname} {
    variable TARGET

    if [info exists TARGET] {
	if {[file pathtype $pathname] == "relative"} {
	    set pathname $TARGET/$pathname
	}
    }
    return $pathname
}

proc source_expand {pathname} {
    variable DATADIR
    variable setname

    return $DATADIR/$pathname
}

proc add {what args} {
    variable instfile

    set f [open $instfile a]
    puts $f [concat $what $args]
    close $f
}

proc addreq {what this} {
    variable VARDIR

    regexp {^(.*)-(\d+)\.(\d+)$} $what {} name major minor
    set max [expr 10 + $minor]
    while {$minor < $max} {
	if [file exists $VARDIR/install/$name-$major.$minor] {
	    set found $name-$major.$minor
	    break
	}
	incr minor
    }
    if ![info exists found] {
	return 
    }
    set cmd {}
    foreach line [split [get_file $VARDIR/install/$found] "\n"] {
	foreach {cmd arg} $line break
	if {[string compare $cmd r] == 0 && [string compare $this $arg] == 0} {
	    return
	}
    }
    Log "Adding dependency information: $this needs $found"
    set f [open $VARDIR/install/$what a]
    puts $f [list r $this]
    close $f
}

proc readversion {filename infovar} {
    upvar $infovar info

    set info(pl) 0
    set info(rev) 0
    set info(export) 0
    set info(agent) 0
    if ![file isfile $filename] {
	return 0
    }
    foreach line [split [get_file $filename] "\n"] {
	foreach {cmd v pl rev export} $line break
	switch -- $cmd {
	    version {
		set info(version) $v
		set info(pl) $pl
		set info(rev) $rev
		set info(export) [string compare $export domestic]
		return 1
	    }
	    agent { set info(agent) 1 }
	    a     { set info(agent) 1 }
	}
    }
    return -1
}

# Backwards compatibility...
proc readVERSION {filename infovar} {
    upvar $infovar info

    set found 0
    set info(pl) 0
    set info(rev) 0
    set info(export) 0
    if ![file isfile $filename] {
	return -1
    }
    foreach line [split [get_file $filename] "\n"] {
	if [regexp {^([^#=]+)="?([^"]*)"?$} $line  {} var value] {
	    incr found
	    switch -exact -- $var {
		VERSION { set info(version) $value }
		PATCHLEVEL { set info(pl) $value }
		REV { set info(rev) $value }
		CRYPT_LICENSE {
		    set info(export) [string compare $value domestic]
		}
	    }
	}
    }
    return [expr $found >= 3]
}

# Check if name-0.0 is installed: < 0 : mismatch, 0 : no, > 0 : ok
#
#  info(version)	version found
#  info(pl)		patchlevel
#  info(rev)		revision
#  info(export)		0/1
#  info(upgrade)	0/1 if upgradable
#  info(agent)		module is an agent
#  info(namever)	name-#.#
#
# Different semantics for agents: they never conflict.
#
proc isinstalled {module installedvar thisvar} {
    upvar $installedvar info
    upvar $thisvar this
    variable VARDIR
    variable SRCDIR
    variable APPDIR

    set info(version) 0.0
    set info(upgrade) 0
    set info(pl) 0
    set info(rev) 0
    set info(export) 0
    set info(agent) 0
    set info(namever) {}

    D 4 "isinstalled: $module"
    if {[readversion $SRCDIR/$module/description this] < 0} {
	D 4 "isinstalled: checking VERSION.txt"
	readVERSION $SRCDIR/$module/VERSION.txt this
    }

    regexp {^(.*)-(\d+)\.(\d+)$} $module {} name major minor
    set pattern $name
    append pattern {-[0-9]*.*[0-9]}
    foreach found [glob -nocomplain [file join $VARDIR install $pattern]] {
	D 4 "isinstalled: found $found"
	set info(namever) [file tail $found]
	regexp {^.*-(\d+)\.(\d+)$} $found {} a b
	set info(version) $a.$b
	if {[readversion $found info] < 0} {
	    D 4 "isinstalled: checking installed VERSION.txt"
	    readVERSION $APPDIR/lib/$name-$a.$b/VERSION.txt info
	}
	set conflict -1
	if $info(agent) {
	    D 4 "Is an agent: does not conflict"
	    set conflict 0
	}
	D 4 "isinstalled: pl=$info(pl) rev=$info(rev) export=$info(export)"
	if {$a != $major} {
	    D 4 "isinstalled: $module: other major ($a). Conflict"
	    return $conflict
	}
	if {$b < $minor} {
	    D 4 "isinstalled: $module: older minor (upgrade ok)"
	    set info(upgrade) 1
	    return $conflict
	}
	if {$b > $minor} {
	    D 4 "isinstalled: $module: newer minor"
	    return 1
	}
	if {$info(pl) < $this(pl)} {
	    D 4 "isinstalled: older pl (upgrade ok)"
	    set info(upgrade) 1
	}
	if {$info(pl) == $this(pl) && $info(rev) < $this(rev)} {
	    D 4 "isinstalled: older rev (upgrade ok)"
	    set info(upgrade) 1
	}
	return 1
    }
    D 4 "isinstalled: $module not found"
    return 0
}

proc runscript {scriptname errvar} {
    upvar $errvar err
    variable APPDIR
    variable setname
    variable upgrade

    set err {}
    set file $APPDIR/lib/$setname/install.tcl
    if ![file exists $file] {
	return 1
    }
    Log "Run $scriptname script"

    catch { namespace delete ::CatModule }
    if [catch {source $file} msg] {
	global errorInfo

	set err [list "Failed to source install script $msg"]
	Log "Source failed: $msg {$errorInfo}"
	return 0
    }
    foreach v {APPDIR VARDIR ETCDIR SRCDIR DATADIR BRAND SHORTNAME MOD} {
	variable $v
	if [info exists $v] {
	    set info($v) [set $v]
	}
    }
    set info(upgrade) 0
    if {$scriptname == "install"} {
	if $upgrade(active) {
	    set info(upgrade) 1
	    set info(upgrade_from) $upgrade(from)
	    set info(upgrade_to) $upgrade(to)
	}
    }
    if [catch {CatModule::Install $scriptname info} msg] {
	Log "Script failed: $msg"
	set err [list $msg]
	return 0
    }
    if [info exists info(ERROR)] {
	set err [list $info(ERROR)]
	if [info exists info(ERRORINFO)] {
	    lappend err $info(ERRORINFO)
	}
	return 0
    }
    if [info exists info(MESSAGE)] {
	set err [list $info(MESSAGE)]
    }
    Log "Module script done"
    return 1
}

proc create_dirs {errorvar} {
    upvar $errorvar err
    variable VARDIR
    variable ETCDIR
    variable APPDIR

    foreach dir [list $VARDIR $ETCDIR $APPDIR] {
	if [file isdirectory $dir] continue
	Log "mkdir $dir"
	if [catch {file mkdir $dir} msg] {
	    err err MKDIR $msg $dir
	    return 0
	}
    }
    return 1
}

proc varexpand {data {varlist {}}} {
    variable APPDIR
    variable VARDIR
    variable ETCDIR
    variable BRAND
    variable SHORTNAME
    variable MOD
    variable MODULENAME
    variable EXPORT

    set CL domestic
    if $EXPORT { set CL export; }
    regsub -all {[<]<INSTALL:VARDIR>>} $data [file nativename $VARDIR] data
    regsub -all {[<]<INSTALL:ETCDIR>>} $data [file nativename $ETCDIR] data
    regsub -all {[<]<INSTALL:APPDIR>>} $data [file nativename $APPDIR] data    
    regsub -all {[<]<INSTALL:EXPORT>>} $data $EXPORT data
    regsub -all {[<]<INSTALL:CRYPT_LICENSE>>} $data $CL data
    regsub -all {[<]<INSTALL:BRAND>>} $data $BRAND data
    regsub -all {[<]<INSTALL:SYSTEM>>} $data hpux-11.0 data
    regsub -all {[<]<INSTALL:CATVERSION>>} $data 5.2 data
    regsub -all {[<]<INSTALL:SHORTNAME>>} $data $SHORTNAME data
    if [info exists MOD] {
	regsub -all {[<]<INSTALL:MODULE>>} $data $MOD data
    }
    set name $MOD
    if [info exists MODULENAME] {
	set name $MODULENAME
    }
    regsub -all {[<]<INSTALL:MODULENAME>>} $data $name data
    foreach {name value} $varlist {
	regsub -all "\[<\]<INSTALL:$name>>" $data $value data
    }
    return $data
}

proc install_agent {} {
    variable MOD
    variable ETCDIR
    variable APPDIR
    variable upgrade

    Log "Agent-specific installation tasks:"

    lappend list $ETCDIR/services/$MOD.cfg
    lappend list $APPDIR/lib/service.cfg
    lappend list $APPDIR/lib/$MOD/port.cfg
    lappend list $APPDIR/lib/port.cfg
    foreach {target template} $list {
	if [file exists $target] continue
	if ![file exists $template] continue
	if ![file isdirectory [file dirname $target]] continue
	Log "Install template: $template -> $target"
	add f $target
	set data [get_file $template]
	set data [varexpand $data [list SERVICE $MOD]]
	set f [open $target w]
	puts $f $data
	close $f
    }

    

    
    
    
    

    return 1
}

proc uninstall_agent {file errvar} {
    upvar $errvar err
    

    Log "Agent specific uninstall tasks:"

    foreach line [split [get_file $file] "\n"] {
	foreach {a svc} $line break
	if [string compare $a a] {
	    continue
	}
	set ports [CatConf::GetConfiguredPorts info $svc]
	if [llength $ports] {
	    err err PORTS "There are still ports configured for $svc"
	    return 0
	}
	
	break
    }
    return 1
}

proc showlicense {file module dont_ask} {
    global env

    if ![file isfile $file] {
	return -1
    }
    if $dont_ask {
	puts stderr "Bypassing license query - license accepted."
	puts stderr "Read license in $file ."
	return 1
    }


    if [info exists env(LINES)] {
	set lines [expr $env(LINES) - 3]
    } else {
	set lines 21
    }

    set n 0
    foreach line [split [get_file $file] "\n"] {
	incr n
	puts stderr $line
	if {$n == $lines} {
	    set n 0
	    puts -nonewline stderr "Press return to continue..."
	    gets stdin answer
	    if {[string compare $answer "y"] == 0} {
		return 1
	    }
	}
    }

    while 1 {
	puts -nonewline stderr \
		"License for $module accepted? (y/n) "
	gets stdin answer
	set answer [string trim $answer]
	switch -glob -- $answer {
	    {[yY]} {
		return 1
	    }
	    {[nN]} {
		return 0
	    }
	    default {
		puts stderr "\nPlease answer \"y\" or \"n\"\n"
	    }
	}
    }
}

variable upgrade
set upgrade(active) 0

proc prepare_upgrade {from to} {
    variable upgrade
    variable VARDIR

    Log "Prepare upgrade $from $to"
    set upgrade(from) $from
    set upgrade(to) $to
    set upgrade(active) 1

    
    set file $VARDIR/install/$from
    D 3 "prepare_upgrade: mv $file $file.upgrade"
    file rename -force -- $file $file.upgrade

    
    foreach line [split [get_file $file.upgrade] "\n"] {
	foreach {cmd path} $line break
	set msg ""
	switch -glob -- $cmd {
	    d  {
		if [catch {set f [open $path/.upgrade w]; close $f}] {
		    set msg ": $msg"
		}
		D 3 "prepare_upgrade: touch $path/.upgrade$msg"
	    }
	    {[bpPfFs]} {
		if [catch {file rename -force -- $path $path.upgrade} msg] {
		    set msg ": $msg"
		}
		D 3 "prepare_upgrade: mv $path $path.upgrade$msg"
	    }
	}
    }
    return 0
}

proc end_upgrade {success} {
    variable VARDIR
    variable upgrade

    if !$upgrade(active) {
	return 0
    }
    set upgrade(active) 0

    if $success {
	Log "Upgrade complete"
    } else {
	Log "Upgrade failed - abort and restore"
    }

    set fromfile $VARDIR/install/$upgrade(from).upgrade

    set copy 0
    foreach line [split [get_file $fromfile] "\n"] {
	foreach {cmd args} $line break
	switch -glob -- $cmd {
	    ok {
		incr copy
	    }
	    r {
		if $success {
		    D 3 "copy requirement: $args"
		    add r $args
		}
	    }
	    d {
		rm $args/.upgrade
	    }
	    {[bpPfFs]} {
		if $success {
		    rm $args.upgrade
		} else {
		    rm $args
		    catch {file rename -force -- $args.upgrade $args}
		}
	    }
	    default {
		if {$copy && $success} {
		    D 3 "copy $cmd: $args"
		    add $cmd $args
		}
	    }
	}
    }
    if $success {
	rm $fromfile
    } else {
	rm $VARDIR/install/$upgrade(from)
	catch {file rename -force -- $fromfile $VARDIR/install/$upgrade(from)}
    }
    return 1
}

#----------------------------------------------------------------------
# Install-file functions

# target dir
# -- prepend all relative targets with this directory
#
proc target {directory args} {
    variable TARGET

    Log "target $directory"
    set TARGET $directory
}

# fileset name description
# -- (shoud be the same as the package name).
#
proc fileset {name description args} {
    variable VARDIR
    variable EXPORT
    variable instfile
    variable setname $name
    variable status
    variable MOD $name
    variable MODULENAME $description
    
    Log "fileset $name $description"
    if ![info exists VARDIR] {
	error [list NOVARDIR "vardir not set"]
    }
    if [catch {file mkdir $VARDIR/install} msg] {
	error [list MKDIR "$msg" $VARDIR/install]
    }

    set nok [isinstalled $name info me]

    set EXPORT 0
    if [info exists $me(export)] {
	set EXPORT 1
    }

    if {$nok < 0} {
	error [list CONFLICT "conflicting version already installed" $match]
    } elseif {$nok > 0 && !$info(upgrade)} {
	error [list ALREADY "$name is already installed" $match]
    }
    set instfile $VARDIR/install/$setname
    set status cleanup
    add fileset $name $description
}

# require name
# -- require other modules
#
proc require {name args} {
    variable setname
    variable reqlist
    variable status cleanup

    Log "require $name"
    if {[isinstalled $name ignore0 ignore1] <= 0} {
	error [list REQUIRE "required module not installed" $name]
    }
    add require $name
    lappend reqlist $name
}

# d directory
# -- create a directory. Do not add the directory to the remove list
#    if it already exists.  Still not correct,but at least not worse.
#
proc d {directory args} {
    Log "d $directory"
    set dir [target_expand $directory]
    set exists [file isdirectory $dir]
    set upgrade [file exists $dir/.upgrade]
    file mkdir $dir
    if {$upgrade || !$exists} {
	add d $dir
    } else {
	Log "$dir: directory already exists (will not remove on uninstall)"
    }
}

# n directory
# -- need a directory, which should already exist, bit if it does not
#    it will be created.  It will not be removed on uninstallation
#
proc n {directory args} {
    Log "n $directory"
    set dir [target_expand $directory]
    if [file isdirectory $dir] {
	Log "exists already, ok"
    } else {
	file mkdir $dir
    }
}

# p program source
# -- install a program, library or similar executable, binary thing
#
proc p {program source args} {
    Log "p $program $source"
    set prog [target_expand $program]
    set src [source_expand $source]
    if ![file exists $src] {
	if [file exists "$src.exe"] {
	    set source "$src.exe"
	}
    }
    rm $prog
    file copy -force -- $src $prog
    catch {file attributes $prog -permissions 0755}
    add p $prog
}

# f file source
# -- install a text file
#
proc f {filename source args} {
    Log "f $filename $source"
    set file [target_expand $filename]
    set data [get_file [source_expand $source]]
    rm $file
    set f [open $file w]
    puts $f $data
    close $f
    add f $file
}

# F file source
# -- install a text file, replacing occurences of "<<INSTALL:???>>"
#    for a number of symbols:
#
#	BRAND		- Long Brand Name
#	SHORTNAME	- Short Brand Name
#	MODULENAME	- File set description
#	MODULE		- File set name
#	APPDIR, VARDIR, ETCDIR
#
proc F {filename source args} {
    Log "F $filename $source"
    set file [target_expand $filename]
    set data [get_file [source_expand $source]]
    set result {}
    set data [varexpand $data]
    rm $file
    set f [open $file w]
    puts $f $data
    close $f
    add f $file
}

# P file source
# -- install an executable text-file
#
proc P {filename source args} {
    Log "P $filename $source"
    set file [target_expand $filename]
    set data [get_file [source_expand $source]]
    set result {}
    set data [varexpand $data]
    rm $file
    set f [open $file w]
    puts $f $data
    close $f
    catch {file attributes  $file -permissions 0755}
    add p $file
}

# b file source
# -- install a binary file
#
proc b {filename source args} {
    Log "b $filename $source"
    set file [target_expand $filename]
    set src [source_expand $source]
    rm $file
    file copy -force -- $src $file
    add b $file
}

# s link target
# -- create a symbolic link
#
proc s { link target args } {
    Log "s $link $target"
    
    set link [target_expand $link]
    rm $link
    exec ln -s $target $link
    add s $link
    
    
}

# Ignore, not used during installation
#
proc i { args } {
    Log "i $args (ignore)"
}

# Ignore, not used right now
#
proc system { args } {
    Log "system $args"
}

# Mostly ignored. Use for "patchlevel" information
#
proc version { args } {
    variable EXPORT

    Log "version $args"
    eval add version $args
    set EXPORT [string compare [lindex $args 3] domestic]
}

# When installaing, note that we may need a "port" template
# and maybe a service template too.
#
# When uninstalling, check that there are no active contigurations
#
proc agent { args } {
    variable isagent 1
    variable MOD

    Log "agent $args"

    add a $MOD
}


# Names...
#
proc brand {shortname longname} {
    variable SHORTNAME $shortname
    variable BRAND $longname

    Log "brand $shortname $longname"
}
#----------------------------------------------------------------------
# Public functions

# Install
# -- Install a module, given its description file
#
proc Install {module errorvar dont_ask upgrade_from} {
    variable SRCDIR
    variable TARGET
    variable DATADIR
    variable MODULENAME	{}
    variable reqlist
    variable setname
    variable status	ignore
    variable isagent 0
    upvar $errorvar err

    Log "Install $module:"
    set setname $module
    set err {}
    if ![info exists SRCDIR] {
	err err DIRNOTSET "directory not defined" SRCDIR
	return 0
    }
    if ![create_dirs err] {
	return 0
    }
    catch { unset TARGET }

    if ![showlicense $SRCDIR/$module/LICENSE.txt $module $dont_ask] {
	return 0
    }
    set file $SRCDIR/$module/description
    set reqlist {}
    set DATADIR $SRCDIR/$module

    if [string length $upgrade_from] {
	prepare_upgrade $upgrade_from $module
    }

    if [catch {namespace eval [namespace current] [list source $file]} err] {
	if ![proper_error $err] {
	    err err ERROR $err
	}
	return 0
    }
    set status cleanup
    foreach req $reqlist {
	addreq $req $setname
    }
    set ok 1
    set msg {}
    if $isagent {
	if [catch {install_agent} msg] {
	    err err POSTINST $msg
	    set ok 0
	}
    }
    if $ok {
	set ok [runscript install msg]
    }
    if [string length $msg] {
	set err [linsert $msg 0 POSTINST]
    }
    if !$ok {
	return 0
    }
    set now [clock seconds]
    add ok [list $now [clock format $now]]
    set status ok
    register_software install err
    end_upgrade 1

    return 1
}

# InstDir
# -- Set one of the installation directory locations.  If the directory
#    looks like "<<*", an environment variable is tried instead
#
proc InstDir {which directory {errvar {}}} {
    if [string length $errvar] {
	upvar $errvar err
    }
    if [string match <<* $directory] {
	global env

	if ![info exists env(CAT_$which)] {
	    error [list DIRNOTSET "directory not defined" $which]
	}
	set directory $env(CAT_$which)
    }
    switch -regexp -- $which {
	APPDIR|ETCDIR|VARDIR {
	    
	}
	SRCDIR {
	    if ![file isdirectory $directory] {
		error [list NODIR "directory does not exist" $directory]
	    }
	}
	default {
	    return 0
	}
    }
    variable $which $directory
    return 1
}

# Uninstall
# -- Uninstall a package
#
proc Uninstall {name errorvar {force 0}} {
    variable VARDIR
    variable ETCDIR
    variable APPDIR
    variable setname $name
    variable MOD $setname
    upvar $errorvar err

    Log "Uninstall $name:"
    set err {}
    set file $VARDIR/install/$name
    if ![file exists $file] {
	err err NOTINST "$name not installed"
	return 0
    }
    if {![file writable $file] || ![file writable $VARDIR/install]} {
	err err NOACCESS "need write access"
	return 0
    }

    # Make sure there are no modules depending
    # on the module we want to uninstall.
    #
    set list {}
    foreach line [split [get_file $file] "\n"] {
	set list [linsert $list 0 $line]
    }
    set cmd {}

    foreach line $list {
	foreach {cmd arg} $line break
	switch -glob -- $cmd {
	    r {
		if [file exists $VARDIR/install/$arg] {
		    err err REQUIRED "in use by installed module" $arg
		    return 0
		}
	    }
	}
    }

    if ![uninstall_agent $file err] {
	return 0
    }

    killer init

    set ok [runscript uninstall msg]
    if [string length $msg] {
	set err [linsert $msg 0 PREUNINST]
    }
    if {!$ok && !$force} {
	return 0
    }

    set regkey {}
    foreach line $list {
	foreach {cmd arg} $line break
	switch -glob -- $cmd {
	    {[bfpsd]} {
		
		rm $arg
		
	    }
	    regkey {
		set regkey $arg
	    }
	    default {}
	}
    }
    rm $file

    Log "Uninstall: deleting empty directories:"
    foreach dir [list $APPDIR $ETCDIR $VARDIR/install $VARDIR] {
	catch {file delete -- $dir} ;# "Soft" remove. Do *not* use "rm"!
	if [file exists $dir] {
	    Log "$dir"
	} else {
	    Log "$dir not empty"
	}
    }
    if [string length $regkey] {
	register_software $regkey err
    }
    return 1
}

# InstallInfo
# -- get assorted information from a module, such as the list of
#    required modules, which of these have not been installed etc.
#
#    requires:    list of required modules
#    missing:     subset of which that are not installed
#    name:        fileset name
#    description: one-line description
#    installed:   -1/0/1
#
#    version	  version already installed
#    namever      name-#.# already installed
#    pl		  ditto patchlevel
#    rev	  ditto revision
#    export	  ditto export (0/1)
#    upgrade	  0/1 if upgradable
#    agent	  installed module is an agent
#
#    this{export,version,pl,rev} - information on current module
#
#
proc InstallInfo {file arrayvar} {
    upvar $arrayvar data
    variable SRCDIR
    
    set data(namever) {}
    D 4 "InstallInfo $file"
    catch {unset data}
    if [file isdirectory $file] {
	if [file exists $SRCDIR/$file/description] {
	    set file $SRCDIR/$file/description
	} else {
	    D 4 "InstallInfo: not found"
	    return 0
	}
    }
    set cmd {}
    set arg0 {}
    set arg1 {}
    foreach line [split [get_file $file] "\n"] {
	if [string match #* $line] continue
	foreach {cmd arg0 arg1} $line break
	if ![string length $cmd] continue
	switch -exact -- $cmd {
	    require {
		D 4 "InstallInfo: requires $arg0"
		lappend data(requires) $arg0
	    }
	    fileset {
		D 4 "InstallInfo: fileset $arg0 $arg1"
		set data(name) $arg0
		set data(description) $arg1
	    }
	}
    }
    if ![info exists data(name)] {
	D 4 "InstallInfo: no name. Not a real file?"
	return 0
    }
    set data(missing) {}
    if [info exists data(requires)] {
	foreach req $data(requires) {
	    if {[isinstalled $req ignore0 ignore1] <= 0} {
		D 4 "InstallInfo: required module $req missing"
		lappend data(missing) $req
	    }
	}
    } else {
	set data(requires) {}
    }
    set data(installed) [isinstalled $data(name) data this]
    set data(thisexport) $this(export)
    set data(thisversion) $this(version)
    set data(thispl) $this(pl)
    set data(thisrev) $this(rev)

    return 1
}

# Check if a module is installed
proc IsInstalled {name {infovar {}}} {
    if [string length $infovar] {
	upvar $infovar info
    }
    if {[isinstalled $name foo bar] > 0} {
	D 3 "IsInstalled $name -> 1"
	return 1
    }

    D 3 "IsInstalled $name -> 0"
    return 0
}

# CleanupInstall
# -- uninstall partial installation
#
proc CleanupInstall {} {
    variable status
    variable setname

    if {[string compare $status cleanup] == 0} {
	D 3 "CleanupInstall: uninstalling $setname"
	if [catch {Uninstall $setname err}] {
	    D 3 "CleanupInstall: uninstall: $err"
	}
    }
    end_upgrade 0
}

# List available module in a directory (default SRCDIR)
#
proc ListInstallables { {dir {}} } {
    variable SRCDIR

    if ![string length $dir] {
        set dir $SRCDIR
    }
    set names {}
    foreach dir [glob -nocomplain [file join $dir *-*]] {
        if ![file exists $dir/description] continue
        if ![regexp {.*/([^/]+-\d+\.\d+)$} $dir {} name] continue
        lappend names $name
    }
    D 2 "ListInstallables: $names"
    return $names
}

# List installed modules, and get information on them
#
proc ListInstalled { {arrayvar -} {pattern *} } {
    variable VARDIR
    if [string compare $arrayvar -] {
	upvar $arrayvar data
    }

    set result {}
    set cmd {}
    set arg0 {}
    set arg1 {}
    foreach m [glob -nocomplain [file join $VARDIR install $pattern]] {
	if ![file isfile $m] continue
	if ![regexp {.*/([^/]+-\d+\.\d+)$} $m {} name] continue
	lappend result $name
	foreach line [split [get_file $m] "\n"] {
	    foreach {cmd arg0 arg1} $line break
	    switch -exact -- $cmd {
		fileset {
		    set data($name,description) $arg1
		}
		r {
		    lappend data($name,required) $arg0
		}
		require {
		    lappend data($name,requires) $arg0		    
		}
	    }
	}
    }
    D 2 "ListInstalled: $result"
    return $result
}

}; # end namespace CatInst
