#
# $Id: confmenus.tcl,v 1.32 2000/05/08 11:02:38 elias Exp $
#

catch {namespace import Menu::*}
catch {namespace import IWrap::*}
catch {namespace import CatInfo::*}

proc Hdr {string} {
    puts stderr "\n\n$string"
    puts [DashLine 70 -]
}

proc PressReturn {} {
    GetAnswer "\n(Press return)"
}


#----------------------------------------------------------------------
# Main Menu
#


# Print a corny menu, and switch into sub-menus, few of which are actual
# menus.  In this context a "menu" is anything interactive.
#
proc MainMenu {} {
    global Menus

    Menu -repeat 1 -after ConfFlush \
	    -title "Key  Command   Description" \
	    -menu {
	a AddMenu      "add       Add a new agent configuration"
	d DelMenu      "del       Delete an agent configuration"
	l ListMenu     "list      List configurations"
	h HostPsdMenu  "host VC   Install Agent host virtual card"
	c CaMenu       "ca        Install CA certificates"
	b BoksMenu     "boks      Set BoKS domain and node key"
	e ElsMenu      "els       Set ELS server(s)"
	s ShowMenu     "show      Show global settings"
	q break        "quit"
	
    }
}

#----------------------------------------------------------------------
# Show
#
proc ShowMenu {} {
    Hdr "Current Configuration"
    ShowCurrentConfig
    PressReturn
}


#----------------------------------------------------------------------
# List
#
proc ListMenu {} {
    global Menus

    Hdr "Configured Ports"
    if ![ShowConfiguredPorts] {
	Msg "\nNo ports configured"
    }
    PressReturn
}

#----------------------------------------------------------------------
# Add
#
proc AddMenu {} {
    global Menus
    variable map

    Hdr "Configure Port"
    set protolist $Menus(protolist)
    set services [GetInstalledServices sdata]
    if ![llength $services] {
	Msg "There are no installed services to configure"
	PressReturn
	return
    }
    foreach s $services {
	set n $sdata($s,name)
	lappend list $n [format "%-15s %s" $n $sdata($s,description)]
    }
    PromptList add Menus data {
	PromptFor service m "Select service to configure:" \
		-proc AddMenuCheck -data $list

	
	PromptFor inetd y "Wrap entry started by inetd?" -default n \
		-proc AddMenuCheck -disables {port server} -enables itag
	PromptFor itag z "Tag in inetd.conf:" \
		-prog AddMenuCheck
	

	

	PromptFor port n "Port to listen to:" \
		-proc AddMenuCheck
	PromptFor proto m "Security Protocol:" -data $protolist \
		-proc AddMenuCheck
	PromptFor method z "Authentication Method:" \
		-proc AddMenuCheck
	PromptFor server z "Server to connect to:" \
		-proc AddMenuCheck
	PromptFor descr z "Description for this port:"

	for { set n 0 } { $n < 20 } { incr n 1 } {
	    PromptFor custom_$n z {} -off 0
	} 
    } {
	set service $data(service,value)
	set svc [InfoName $service]
	set port $data(port,value)
	set proto $data(proto,value)
	set server $data(server,value)
	set method $data(method,value)
	set descr $data(descr,value)
	
	set hook 0
	set inetd $data(inetd,value)
	set itag $data(itag,value)
	
	

	if ![Startup] {
	    Msg "Unable to start cat daemon"
	    PressReturn
	    return
	}

	
	if $inetd {
	    set args [list $itag $service $proto $method auto $descr]
	    if ![eval ConfigureInetdPort $args] {
		Msg "Failed to configure port"
		PressReturn
		return
	    }
	}
	

	

	if {!$inetd && !$hook} {
	    if {[string length $server] > 0 && [string compare $server {-}]} {
		set host [info hostname]
		if {[string compare $host:$port $server] == 0} {
		    Msg "Server is $server, agent will listen to $host:$port"
		    if ![GetYesNo "Do you know what you are doing?" n] {
			return
		    }
		}
		if ![HostLooksReachable $server] {
		    set s "Reachability test: problems contacting \"$server\""
		    if ![GetYesNo "$s\nContinue?" y] {
			return
		    }
		}
	    }
	    set args [list $port $service $proto $method $server auto $descr]
	    if ![eval ConfigurePort $args] {
		Msg "Failed to configure port"
		PressReturn
		return 
	    }
	}
	set lib [CatDir APP]/lib
	if [file exists $lib/$service/config.tcl] {
	    catch { namespace delete ::CatModule }
	    catch { source $lib/$service/config.tcl } 

	    if [catch { CatModule::ModuleSpecificConf data }] {
		Msg "Error configuring module specific data"
		DeletePort $port
		PressReturn
		return 
	    }
	}
	set no 0
	foreach tag [array names data custom_*,enabled] {
	    if { $data($tag) } {
		set path $data(custom_$no,path)
		set value $data(custom_$no,value)
		
		if { $data(port,value) > 0 } {
		    set port $data(port,value)
		} else {
		    
		    set port [eval GetServByName $data(itag,value)]
		    
		    
		}
		Set .tcp.s$port.$path $data(custom_$no,value)
	    }
	    incr no
	}
    }
}

namespace eval ::ParamConfig { ;# 

proc DoParamConfig { data } {
    upvar $data gdata
    
    set lib [CatDir APP]/lib
    set service $gdata(service,value)

    if [file exist $lib/$service/custom.tcl] {
	if [catch { source $lib/$service/custom.tcl }] {
	    Msg "Failed set agent specific data, check your custom.tcl file."
	    set gdata(abort) 1
	    PressReturn
	    return 1
	}
    }
}

} ;# end namespace ParamConfig
 
proc AddMenuCheck {arrayvar tag value} {
    upvar $arrayvar data

    switch -regexp $tag {
	service {
	    set svc [InfoName $value]
	    if [Get .services.$svc.default.authmethod s] {
		set data(method,default) $s
	    }
	    if [Get .services.$svc.default.server s] {
		if [string match tcp:* $s] {
		    set s [string range $s 4 end]
		}
		set data(server,default) $s
	    }
	    if [Get .services.$svc.default.port s] {
		set data(port,default) $s
	    }
	    if [Get .services.$svc.default.inetd s] {
		set data(itag,default) $s
		set data(inetd,default) y
	    }
	    if [Get .services.$svc.default.server serv] {
		switch -- $serv {
		    - {
			# Server (doesn't connect to anything)
			
			set data(itag,enabled) 0
			set data(itag,value) $data(itag,default)
			set data(inetd,enabled) 0
			if {[string length $data(itag,value)] > 0} {
			    set data(inetd,value) 1
			} else {
			    set data(inetd,value) 0
			}
			
			
			set data(server,enabled) 0
			set data(server,value) {}
		    }
		    default {
		    }
		}
	    }
	    ::ParamConfig::DoParamConfig data
	}

	
	inetd {
	    if $value {
		if ![IWrap::CanAccess] {
		    Msg "Must be root to modify /etc/inetd.conf"
		    set data(abort) 1
		    PressReturn
		}
	    }
	}
	itag {
	    set list [IWrap::Find $value]
	    if ![llength $list] {
		Msg "Service \"$value\" does not exist in inetd.conf"
		return 0
	    }
	}
	

	

	bindport|port {
	    if ![LooksLikePort $value] {
		puts stderr "Bad port number."
		return 0
	    }
	    if [GetPortInfo p $value] {
		set s $p(service)
		if [string compare $p(service) $data(service,value)] {
		    set str "Port $value is currently configured for $s\n"
		    append str "Changing values will disable $s on this port."
		    set prompt "Continue?"
		    set y n
		} else {
		    set str "This port is already configured for $s"
		    set prompt "Reconfigure values?"
		    set y y
		}
		Msg $str
		if ![GetYesNo $prompt $y] {
		    set data(abort) 1
		    return 1
		}
	    }
	}
	proto {
	    if {[string compare $value plain] == 0} {
		set data(method,enabled) 0
	    } else {
		if ![Get .global.hostpsd.file s] {
		    set s "NOTICE:\n"
		    append s "  There is no Host Psd configured yet."
		    append s "  Encryption protocols will not work\n"
		    append s "  until one has been configured.\n"
		    Msg $s
		}
	    }
	}
	server {
	    if ![LooksLikeHost $value 1] {
		if [LooksLikeHost $value] {
		    set port $data(port,value)
		    puts stderr "Using server port $port"
		    set data($tag,value) $value:$port
		} else {
		    puts stderr "Expected <servername-or-address>:<port>"
		    return 0
		}
	    }
	}
    }
    return 1
}

#----------------------------------------------------------------------
# Delete
#
proc DelMenu {} {
    while 1 {
	Hdr "Delete Port Setup"
	if ![ShowConfiguredPorts data] {
	    Msg "There are no ports configured\n"
	    PressReturn
	    return
	}
	puts [DashLine 79 -]
	set port [GetAnswer "Select a port to remove (q to return): "]
	if [string match {[qQ]*} $port] {
	    return
	}
	if ![info exists data($port,service)] {
	    Msg "Port not configured.\n"
	    continue
	}
	break
    }
    GetPortInfo p $port
    set prompt "This will disable $p(service) on port $port. Continue?"
    if [GetYesNo $prompt y] {
	DeletePort $port
    }
}

#----------------------------------------------------------------------
# HostPSD
#
proc HostPsdMenu {} {
    global Menus

    Hdr "Install Agent host virtual card and/or Set virtual card Password"
    PromptList hostpsd Menus data {
	PromptFor dofile y "Install a new Agent host virtual card?" \
		-default n -enables {file pswf}
	PromptFor file s "Virtual card File:"
	PromptFor pswf s "Virtual card Password file (leave empty to be prompted):"
    } {
	if ![ConfHostPSD $data(file,value) $data(pswf,value)] {
	    PressReturn
	}
    }
}

#----------------------------------------------------------------------
# BoKS
#
proc BoksMenu {} {
    global Menus

    GetBoksInfo info
    set domain 6500
    if $info(domain,set) {
	set domain $info(domain)
    }

    PromptList boks Menus data {
	PromptFor setkey y "Set BoKS Node key?" -default n -enables keyfile
	PromptFor keyfile s \
		"Password from file\n  (leave empty to be prompted):" \
		-proc BoksMenuCheck
	PromptFor domain n "BoKS Domain port:" -default $domain \
		-proc BoksMenuCheck
	PromptFor setaddrs y "Set server address list?" \
		-default n -enables servers
	PromptFor servers s "Server list:" -value $info(servers)
    } {
	if {$domain != $data(domain,value)} {
	    if ![ConfBoks domain $data(domain,value)] return
	}
	if $data(setkey,value) {
	    if ![ConfBoksNodekey $data(keyfile,value)] {
		PressReturn
		return
	    }
	}
	if [string compare $info(servers) $data(servers,value)] {
	    if ![ConfBoks servers $data(servers,value)] return
	}
	if ![TryBoks] {
	    Msg "Reachability test: unable to contact BoKS server"
	    PressReturn
	}
	Set .global.start.logger.start on
    }
}

proc BoksMenuCheck {arrayvar tag value} {
    upvar $arrayvar d

    switch $tag {
	domain {
	    if ![LooksLikePort $value 1025] {
		Msg "Bad port number (default is 6500)"
		return 0
	    }
	}
	servers {
	    set servers {}
	    foreach s [split $value ", \t"] {
		if ![LooksLikeHost $s] {
		    Msg "Expected comma-separated list of hosts"
		    return 0
		}
		lappend servers s
	    }
	    set servers [join $servers ,]

	    # If had a list, and answer is empty, ask if it should
	    # be cleared...
	    set o [string length $d($tag,oldvalue)]
	    set n [string length $value]
	    if {$o && !$n} {
		if ![GetYesNo "Clear the BoKS server list?" "n"] {
		    set d(abort) 1
		    return 1
		}
	    }
	}
	keyfile {
	    if [string length $value] {
		if ![file exist $value] {
		    Msg "File $value does not exist"
		    return 0
		}
	    }
	}
    }
    return 1
}

#----------------------------------------------------------------------
# ELS
#
proc ElsMenu {} {
    Hdr "ELS (Event Logging System) Setup"

    GetELSInfo els

    PromptList els {} data {
	PromptFor servers s "ELS Servers:" -proc ElsMenuCheck \
		-value $els(servers)
    } { 
	ConfELS $data(servers,value)

	Set .global.start.logger.start on
    }
}

# Verify that server list looks sane
#
proc ElsMenuCheck {arrayvar tag value} {
    upvar $arrayvar d

    set list {}
    foreach s [split $value ", \t"] {
	set s [string trim $s]
	if ![string length $s] continue
	if ![string match *:* $s] {
	    append s :2479
	}
	if ![LooksLikeHost $s 1] {
	    Msg "Expected a comma-separated list of <server>:<port> ..."
	    return 0
	}
	lappend list $s
    }
    if ![llength $list] {
	puts stderr "Setting an empty server list disables logging to ELS"
	if ![GetYesNo "Continue" "n"] {
	    set d(abort) 1
	    return 1
	}
    }
    set servers [join $list ,]
    set d($tag,value) $servers

    return 1
}

#----------------------------------------------------------------------
# CA Certificate Mgmt
#
proc CaMenu {} {
    Menu -repeat 1 -menu {
	i CatConf::CaMenuImport  "Import a CA certificate"
	r CatConf::CaMenuDel	"Remove a CA certificate"
	l CatConf::CaMenuList	"List currently imported CA certificates"
	q break		"Return to previous menu"
    }
}
