#
# $Id: getopt.tcl,v 1.3 1999/12/14 10:00:52 mikko Exp $
# Copyright 1999 Security Dynamics
#
# getopt --
#
#  Read argv.  The "args" is a standard getopt specifier (a:bc:d etc)
#  The error variable is something suitable (?) to print as
#  error message (if it has a non-zero length, there was an error).
#
#  The option array can be set to a bunch of variables
#  that will be loaded with the argument data:
#
#   opts(<char>_v) variable	<- int (# of times it appeared)
#   opts(<cahr>_V) variable	<- string
#
#  On output, this array will contain one element per option that
#  was set (containing its value), and one element called "args"
#  conntaining the rest of the arguments.
#
#  Returns argv in a split-up form, should one want to parse it
#  "manually".
#
proc getopt {argv args errorvar {optarray {}}} {
    upvar $errorvar opterr
    if [string length $optarray] {
	upvar $optarray opts
    }
    set opts(args) {}
    set opterr {}

    set list {}
    set arg 0
    set skip 0
    set opt args
    foreach a $argv {
	if {$arg || $skip} {
	    lappend opts($opt) $a
	    lappend list $a
	    set arg 0
	    set opt args
	    continue
	}
	if {![string match -* $a] || $a == "--" || $a == "-"} {
	    lappend list --
	    if {$a != "--"} {
		lappend list $a
		if $arg {
		    lappend opts($opt) $a
		} else {
		    lappend opts(args) $a
		}
	    }
	    incr skip
	    set opt args
	    continue
	}
	set a [string range $a 1 end]
	while {[string length $a] > 0} { 
	    set c [string index $a 0]
	    set a [string range $a 1 end]
	    set i [string first $c $args]
	    if {$i < 0} {
		set opterr "Unknown option: -$c"
		return {}
	    }
	    set opt $c
	    lappend list -$c
	    incr i
	    if {[string index $args $i] == ":"} {
		if {[string length $a] > 0} {
		    lappend opts($c) $a
		} else {
		    incr arg
		}
		break
	    } else {
		if [info exists opts($c)] {
		    incr opts($c)
		} else {
		    set opts($c) 1
		}
		set opt args
	    }
	}
    }
    if $arg {
	set opterr "Option requires an argument: -$opt"
	return {}
    }
    foreach v [array names opts] {
	if {[info exists opts(${v}_v)] && [info exists opts($v)]} {
	    upvar $opts(${v}_v) vv
	    set vv $opts($v)
	}
	if {[info exists opts(${v}_V)] && [info exists opts($v)]} {
	    upvar $opts(${v}_V) vv
	    set vv [lindex $opts($v) [expr [llength $opts($v)] - 1] ]
	}
    }
    return $list
}
